﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Interactivity; 
using System.Collections.Specialized;
using System.Windows;
using System.Collections;
using Infragistics.Controls.Editors;

namespace XamMultiComboEditor
{
    public class SelectedItemsSynchronizationBehavior : Behavior<XamComboEditor>
    {
        private XamComboEditor Combobox
        {
            get
            {
                return AssociatedObject as XamComboEditor;
            }
        }

        public INotifyCollectionChanged SelectedItems
        {
            get { return (INotifyCollectionChanged)GetValue(SelectedItemsProperty); }
            set { SetValue(SelectedItemsProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemsProperty =
            DependencyProperty.Register("SelectedItems",
            typeof(INotifyCollectionChanged),
            typeof(SelectedItemsSynchronizationBehavior),
            new PropertyMetadata(OnSelectedItemsPropertyChanged));


        private static void OnSelectedItemsPropertyChanged(DependencyObject target, DependencyPropertyChangedEventArgs args)
        {
            var collection = args.NewValue as INotifyCollectionChanged;
            if (collection != null)
            {

                SelectedItemsSynchronizationBehavior behavior = target as SelectedItemsSynchronizationBehavior;
                behavior.UnsubscribeFromEvents();
                
                if (behavior.Combobox != null)
                {

                    behavior.Transfer(collection as IList, behavior.Combobox.SelectedItems);
                    
                }

                behavior.SubscribeToEvents();
               
            }
        }

        protected override void OnAttached()
        {
            base.OnAttached();

            UnsubscribeFromEvents();
            Transfer(SelectedItems as IList, Combobox.SelectedItems);
            SubscribeToEvents();

            Combobox.SelectedItems.CollectionChanged += Combobox_SelectionChanged;
        }

        void Combobox_SelectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            UnsubscribeFromEvents();

            Transfer( Combobox.SelectedItems, SelectedItems as IList);

            SubscribeToEvents();
        }

        void ContextSelectedItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            UnsubscribeFromEvents();

            Transfer(SelectedItems as IList, Combobox.SelectedItems);

            SubscribeToEvents();
        }

        private void SubscribeToEvents()
        {
            if (Combobox != null)
                Combobox.SelectedItems.CollectionChanged += Combobox_SelectionChanged;

            if (SelectedItems != null)
            {
                SelectedItems.CollectionChanged += ContextSelectedItems_CollectionChanged;
            }
        }

        private void UnsubscribeFromEvents()
        {
            if (Combobox != null)
                Combobox.SelectedItems.CollectionChanged -= Combobox_SelectionChanged;

            if (SelectedItems != null)
            {
                SelectedItems.CollectionChanged -= ContextSelectedItems_CollectionChanged;
            }
        }

        public void Transfer(IList source, IList target)
        {
            if (source == null || target == null)
                return;

            Stack toRemove = new Stack();

            foreach (var i in target)
            {
                if (!source.Contains(i))
                    toRemove.Push(i);
            }

            while (toRemove.Count > 0)
            {
                target.Remove(toRemove.Pop());
            }


            foreach (var o in source)
            {
                if (!target.Contains(o))
                    target.Add(o);
            }
        }
    }
}
