﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.Editors;
using System.Collections;
using Infragistics.Controls.Editors;
using Infragistics.Windows;
using System.Windows.Controls.Primitives;
using Infragistics.Controls.Editors.Primitives;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Windows.Interactivity;
using System.Diagnostics;
using System.Linq;

namespace XamMultiComboEditor
{
    public class XamMultiSelectionComboEditor : ValueEditor, INotifyPropertyChanged
    {
        #region Members 

        ContentControl _nonEditContainer;

        #endregion

        #region Constructor
        public XamMultiSelectionComboEditor()
        {
            DefaultStyleKey = typeof(XamMultiSelectionComboEditor);
        }

        static XamMultiSelectionComboEditor()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(XamMultiSelectionComboEditor),
                new FrameworkPropertyMetadata(typeof(XamMultiSelectionComboEditor)));
        }

        #endregion

        #region Public Properties

        #region ItemSource


        public IEnumerable ItemSource
        {
            get { return (IEnumerable)GetValue(ItemSourceProperty); }
            set { SetValue(ItemSourceProperty, value); }
        }

        public static readonly DependencyProperty ItemSourceProperty =
            DependencyProperty.Register(
                "ItemSource",
                typeof(IEnumerable),
                typeof(XamMultiSelectionComboEditor),
                new UIPropertyMetadata(null));


        #endregion

        #region DisplayMemberPath


        public string DisplayMemberPath
        {
            get { return (string)GetValue(DisplayMemberPathProperty); }
            set { SetValue(DisplayMemberPathProperty, value); }
        }

        public static readonly DependencyProperty DisplayMemberPathProperty =
            DependencyProperty.Register(
                "DisplayMemberPath",
                typeof(string),
                typeof(XamMultiSelectionComboEditor),
                new UIPropertyMetadata(null));


        #endregion

        #region SelectedValuePath


        public string SelectedValuePath
        {
            get { return (string)GetValue(SelectedValuePathProperty); }
            set { SetValue(SelectedValuePathProperty, value); }
        }

        public static readonly DependencyProperty SelectedValuePathProperty =
            DependencyProperty.Register(
                "SelectedValuePath",
                typeof(string),
                typeof(XamMultiSelectionComboEditor),
                new UIPropertyMetadata(null));


        #endregion

        #region SelectedItem


        public object SelectedItem
        {
            get { return (object)GetValue(SelectedItemProperty); }
            set { SetValue(SelectedItemProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemProperty =
            DependencyProperty.Register(
                "SelectedItem",
                typeof(object),
                typeof(XamMultiSelectionComboEditor),
                new UIPropertyMetadata(
                    null,
                    (o, e) =>
                    {
                        XamMultiSelectionComboEditor editor = o as XamMultiSelectionComboEditor;
                        if (e.NewValue != null)
                        {
                            if (editor.ValueType.GetInterface("IEnumerable") == null) 
                            {
                                if (string.IsNullOrEmpty(editor.SelectedValuePath))
                                    editor.Value = e.NewValue;
                                else
                                    editor.Value = e.NewValue.GetType().GetProperty(editor.SelectedValuePath).GetValue(e.NewValue, null);
                            }
                        }

                    }));


        #endregion

        #region ContentTemplate

        public DataTemplate ContentTemplate
        {
            get { return (DataTemplate)GetValue(ContentTemplateProperty); }
            set { SetValue(ContentTemplateProperty, value); }
        }

        public static readonly DependencyProperty ContentTemplateProperty =
                DependencyProperty.Register(
                "ContentTemplate",
                typeof(DataTemplate),
                typeof(XamMultiSelectionComboEditor),
                new PropertyMetadata(null,
                    (o, e) =>
                    {
                        (o as XamMultiSelectionComboEditor).OnPropertyChanged("ContentTemplate");
                    }));


        #endregion

        #region Drop Down Width


        public double DropDownWidth
        {
            get { return (double)GetValue(DropDownWidthProperty); }
            set { SetValue(DropDownWidthProperty, value); }
        }

        // Using a DependencyProperty as the backing store for DropDownWidth.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty DropDownWidthProperty =
            DependencyProperty.Register(
                "DropDownWidth",
                typeof(double),
                typeof(XamMultiSelectionComboEditor), 
                new UIPropertyMetadata(-1.0));


        #endregion

        #region AllowMultiSelect

        public bool AllowMultipleSelection
        {
            get { return (bool)GetValue(AllowMultipleSelectionProperty); }
            set { SetValue(AllowMultipleSelectionProperty, value); }
        }

        public static readonly DependencyProperty AllowMultipleSelectionProperty =
            DependencyProperty.Register(
                "AllowMultipleSelection",
                typeof(bool),
                typeof(XamMultiSelectionComboEditor),
                new PropertyMetadata(true,
                    (o, e) =>
                    {
                        (o as XamMultiSelectionComboEditor).OnPropertyChanged("AllowMultipleSelection");
                    }));
        

        #endregion

        #region CheckBoxVisibility

        public Visibility CheckBoxVisibility
        {
            get { return (Visibility)GetValue(CheckBoxVisibilityProperty); }
            set { SetValue(CheckBoxVisibilityProperty, value); }
        }

        public static readonly DependencyProperty CheckBoxVisibilityProperty =
            DependencyProperty.Register(
                "CheckBoxVisibility", 
                typeof(Visibility), 
                typeof(XamMultiSelectionComboEditor), 
                new PropertyMetadata(Visibility.Visible,
                         (o, e) =>
                         {
                             (o as XamMultiSelectionComboEditor).OnPropertyChanged("CheckBoxVisibility");
                         }));

        

        #endregion
         
        #region SelectedItemsInternal

        private IEnumerable selectedItems;

        internal IEnumerable SelectedItemsInternal
        {
            get { return selectedItems; }
            set 
            {
                selectedItems = value;
               
                OnPropertyChanged("SelectedItemsInternal"); 
            }
        }
        
        #endregion

        #region SelectedItemsInternalFull

        private List<object> _selectedItemsInternalFull = new List<object>();

        public List<object> SelectedItemsInternalFull
        {
            get { return _selectedItemsInternalFull; }
            set { _selectedItemsInternalFull = value; }
        }


        #endregion

        #endregion

        #region ValueEdtior Members
        public override bool CanEditType(Type type)
        {
            return true;
        }

        public override bool CanRenderType(Type type)
        {
            return true;
        }
        #endregion

        #region Base Overrides

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            ContentControl nonEditContent = GetTemplateChild("nonEditContent") as ContentControl;
            if (nonEditContent != null)
            {
                _nonEditContainer = nonEditContent;
                ResolveNonEditableContent();
            }
        }

        protected override object OnCoerceValue(object val)
        {
            if (ValueType.GetInterface("IEnumerable") != null)
            {
                if (val != SelectedItemsInternal)
                {
                    if(SelectedItemsInternal != null
                        && SelectedItemsInternal is INotifyCollectionChanged)
                        (SelectedItemsInternal as INotifyCollectionChanged).CollectionChanged -= SelectedItemsInternal_CollectionChanged;

                    SelectedItemsInternal = (IEnumerable)val;

                    if (SelectedItemsInternal != null
                        && SelectedItemsInternal is INotifyCollectionChanged)
                        (SelectedItemsInternal as INotifyCollectionChanged).CollectionChanged += SelectedItemsInternal_CollectionChanged;
                }
            }
            else 
            {
                if (string.IsNullOrEmpty(SelectedValuePath) || ItemSource == null)
                    SelectedItem = val;
                else
                {
                    var querySource = ItemSource.Cast<object>();

                    var item = querySource.Where(obj => obj.GetType().GetProperty(SelectedValuePath).GetValue(obj, null).Equals(val)).FirstOrDefault();

                    if (item != null)
                    {
                        SelectedItem = item;
                    }
                }
            }

            ResolveNonEditableContent();

            return base.OnCoerceValue(val);
        }

        protected override void OnEditModeStarted(Infragistics.Windows.Editors.Events.EditModeStartedEventArgs args)
        {
            base.OnEditModeStarted(args);

            Infragistics.Controls.Editors.XamComboEditor combo = Utilities.GetDescendantFromType(this, typeof(Infragistics.Controls.Editors.XamComboEditor), false) as Infragistics.Controls.Editors.XamComboEditor;
            if (combo != null)
            {            
                if (SelectedItemsInternal != null)
                {
                    TransferadInternalToSelected(SelectedItemsInternal as IList, combo.SelectedItems, combo.ItemsSource as IList);                    
                    //TransferadSome(SelectedItemsInternal as IList, combo.SelectedItems);
                } 

                combo.DropDownOpened += (ss, ee) =>
                {
                     

                    Popup popup = Utilities.GetDescendantFromName(ss as DependencyObject, "Popup") as Popup;
                    if (popup != null)
                    {
                        Border border;

                        if ((popup.Child as FrameworkElement).Name == "RootPopupElement")
                            border = popup.Child as Border;
                        else
                            border = Utilities.GetDescendantFromName(popup.Child, "RootPopupElement") as Border;

                        if (border != null)
                        {
                            if (DropDownWidth < 1)
                                return;

                            border.Width = DropDownWidth;
                        }

                    }
                };

                SpecializedTextBox textBox = Utilities.GetDescendantFromType(combo, typeof(SpecializedTextBox), false) as SpecializedTextBox;
                if (textBox != null)
                {
                    textBox.Dispatcher.BeginInvoke(new Action(() =>
                        {
                            textBox.Focus();
                            textBox.SelectAll();
                        }), System.Windows.Threading.DispatcherPriority.Background, null);
                }
            }      

             
        }

        void SelectedItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            
        }

        protected override void OnEditModeEnding(Infragistics.Windows.Editors.Events.EditModeEndingEventArgs args)
        {
            base.OnEditModeEnding(args);

            Infragistics.Controls.Editors.XamComboEditor combo = Utilities.GetDescendantFromType(this, typeof(Infragistics.Controls.Editors.XamComboEditor), false) as Infragistics.Controls.Editors.XamComboEditor;
            if (combo != null)
            {
                combo.DropDownOpened -= (ss, ee) => { };

                //Interaction.GetBehaviors(combo).Clear();

                if (SelectedItemsInternal != null)
                {
                    TransferadSelectedToInternal(combo.SelectedItems, SelectedItemsInternal as IList);
                    //TransferadInternalToSelected(SelectedItemsInternal as IList, combo.SelectedItems, combo.ItemsSource as IList);
                    //TransferadSome(combo.SelectedItems, SelectedItemsInternal as IList);
                }
                else
                {
                    var tempCollection = Activator.CreateInstance(ValueType, null, null);

                    if (tempCollection != null
                        && tempCollection is IList)
                    {
                        //TransferadInternalToSelected(SelectedItemsInternal as IList, combo.SelectedItems, combo.ItemsSource as IList);
                        TransferadSelectedToInternal(combo.SelectedItems, tempCollection as IList);
                        //TransferadSome(combo.SelectedItems, tempCollection as IList);
                        

                        Value = tempCollection;
                    }
                } 
            }

          
        }
  
        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        
        public virtual void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        
        #endregion

        #region Private Methods

        private void ResolveNonEditableContent()
        {
            string content = "";
            if (SelectedItemsInternal != null)
            {
                content = "";
                //var itemsToIterate = !string.IsNullOrEmpty(SelectedValuePath) ? SelectedItemsInternalFull : SelectedItemsInternal;
                foreach (var item in SelectedItemsInternalFull)
                {
                    if (string.IsNullOrEmpty(DisplayMemberPath))
                        content += item + ", ";
                    else
                        content += item.GetType().GetProperty(DisplayMemberPath).GetValue(item, null) + ", ";
                }

                if (content.Length > 2)
                    content = content.Substring(0, content.Length - 2);

            }

            if (content.Length == 0)
                content = "no items selected";

            if (_nonEditContainer != null)
                _nonEditContainer.Content = content;
        }

        private void TransferadSome(IList source, IList target)
        {
            if (source == null || target == null)
                return;

            Stack toRemove = new Stack();

            foreach (var i in target)
            {
                if (!source.Contains(i))
                    toRemove.Push(i);
            }

            while (toRemove.Count > 0)
            {
                target.Remove(toRemove.Pop());
            }


            foreach (var o in source)
            {
                if (!target.Contains(o))
                    target.Add(o);
            }
        }

        private void TransferadSelectedToInternal(IList siComnbo, IList siInternal)
        {
            if (siComnbo == null || siInternal == null)
                return;

            if (!string.IsNullOrEmpty(SelectedValuePath))
            {
                Stack toRemove = new Stack();

                foreach (var i in siComnbo)
                {
                    var val = i.GetType().GetProperty(SelectedValuePath).GetValue(i, null);
                    if (!siInternal.Contains(i))
                        toRemove.Push(i);
                }

                while (toRemove.Count > 0)
                {
                    siInternal.Remove(toRemove.Pop());
                }
                SelectedItemsInternalFull.Clear();
                foreach (var i in siComnbo)
                {
                    var toAdd = i.GetType().GetProperty(SelectedValuePath).GetValue(i, null);
                    if (!siInternal.Contains(toAdd))
                    {
                        siInternal.Add(toAdd);
                        
                    }
                    SelectedItemsInternalFull.Add(i);
                }
            }
            else
            {
                TransferadSome(siComnbo, siInternal);

                SelectedItemsInternalFull.Clear();
                foreach (var i in siComnbo)
                    SelectedItemsInternalFull.Add(i);
            }
        }

        private void TransferadInternalToSelected(IList siInternal, IList siCombo, IList comboSource)
        {
            if (siCombo == null || siInternal == null)
                return;

            if (!string.IsNullOrEmpty(SelectedValuePath))
            {
                Stack toRemove = new Stack();
              
                siCombo.Clear();
                SelectedItemsInternalFull.Clear();
                foreach (var i in siInternal)
                {
                    var toAdd = GetComboItemFromValue(comboSource, i);
                    if (toAdd != null && !siCombo.Contains(toAdd))
                    {
                        siCombo.Add(toAdd);
                        SelectedItemsInternalFull.Add(toAdd);
                    }
                }
            }
            else
            {
                TransferadSome(siInternal, siCombo);
                SelectedItemsInternalFull.Clear();
                foreach (var i in siInternal)
                {
                    var toAdd = GetComboItemFromValue(comboSource, i);
                    if (toAdd != null && !siCombo.Contains(toAdd))
                    { 
                        SelectedItemsInternalFull.Add(toAdd);
                    }
                }
            }
        }

        private object GetComboItemFromValue(IList selectedItems, object value)
        {
            if (!string.IsNullOrEmpty(SelectedValuePath))
            {
                foreach (var item in selectedItems)
                {
                    var val = item.GetType().GetProperty(SelectedValuePath).GetValue(item, null);
                    if (value != null && value.Equals(val))
                        return item;
                }
            }
            return null;
        }

        #endregion

        #region Event Handlers

        void SelectedItemsInternal_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {        
            ResolveNonEditableContent();
        }

        #endregion



    }
}
